@@ -8,10 +8,10 @@ use std::sync::Arc;
 
 use crate::context;
 use crate::errctx;
-use crate::error::{add_context, Error, NWFError, Result, WFError};
+use crate::error::{add_context, Error, Result, XmlError};
 use crate::strings::*;
 
-use super::common::{EventMetrics, XMLVersion, XMLNS_XML};
+use super::common::{EventMetrics, XmlVersion, XMLNS_XML};
 use super::raw::{RawEvent, RawQName};
 
 /// Shared namespace URI
@@ -19,7 +19,7 @@ pub type NamespaceName = RcPtr<CData>;
 
 /// Pair of an optional namespace name (URI) and a localpart, commonly used in
 /// element and attribute names.
-pub type ResolvedQName = (Option<NamespaceName>, NCName);
+pub type ResolvedQName = (Option<NamespaceName>, NcName);
 
 /// Wrapper pointer around namespace URIs
 ///
@@ -54,15 +54,15 @@ number of bytes from the input stream used to generate the event.
 pub enum ResolvedEvent {
 	/// The XML declaration.
 	///
-	/// This mirrors [`RawEvent::XMLDeclaration`].
-	XMLDeclaration(
+	/// This mirrors [`RawEvent::XmlDeclaration`].
+	XmlDeclaration(
 		/// Number of bytes contributing to this event.
 		///
 		/// This includes all bytes from the opening `<?` until and including
 		/// the closing `?>`.
 		EventMetrics,
 		/// XML version number
-		XMLVersion,
+		XmlVersion,
 	),
 	/// The start of an XML element.
 	StartElement(
@@ -117,7 +117,7 @@ impl ResolvedEvent {
 	/// Return the [`EventMetrics`] of the event
 	pub fn metrics(&self) -> &EventMetrics {
 		match self {
-			Self::XMLDeclaration(m, ..) => &m,
+			Self::XmlDeclaration(m, ..) => &m,
 			Self::StartElement(m, ..) => &m,
 			Self::EndElement(m, ..) => &m,
 			Self::Text(m, ..) => &m,
@@ -133,7 +133,7 @@ enum State {
 struct ElementScratchpad {
 	phyqname: RawQName,
 	default_decl: Option<NamespaceName>,
-	nsdecl: HashMap<NCName, NamespaceName>,
+	nsdecl: HashMap<NcName, NamespaceName>,
 }
 
 impl ElementScratchpad {
@@ -163,7 +163,7 @@ forward a [`RawEvent::Text`] right after a [`RawEvent::ElementHeadOpen`].
 pub struct NamespaceResolver {
 	ctx: RcPtr<context::Context>,
 	fixed_xml_namespace: NamespaceName,
-	namespace_stack: Vec<(Option<NamespaceName>, HashMap<NCName, NamespaceName>)>,
+	namespace_stack: Vec<(Option<NamespaceName>, HashMap<NcName, NamespaceName>)>,
 	scratchpad: Option<ElementScratchpad>,
 	phyattributes: Vec<(RawQName, CData)>,
 	event_length_accum: usize,
@@ -214,9 +214,7 @@ impl NamespaceResolver {
 				match scratchpad.nsdecl.entry(phyqn.1) {
 					// XML 1.0
 					// Well-formedness constraint: Unique Att Spec
-					Entry::Occupied(_) => {
-						return Err(Error::NotWellFormed(WFError::DuplicateAttribute))
-					}
+					Entry::Occupied(_) => return Err(Error::Xml(XmlError::DuplicateAttribute)),
 					Entry::Vacant(e) => e.insert(self.ctx.intern_cdata(value)),
 				};
 				return Ok(());
@@ -230,7 +228,7 @@ impl NamespaceResolver {
 	}
 
 	fn lookup_prefix<'x>(
-		namespace_stack: &'x Vec<(Option<NamespaceName>, HashMap<NCName, NamespaceName>)>,
+		namespace_stack: &'x Vec<(Option<NamespaceName>, HashMap<NcName, NamespaceName>)>,
 		fixed_xml_namespace: &'x NamespaceName,
 		prefix: Option<&str>,
 	) -> Result<Option<&'x NamespaceName>> {
@@ -259,9 +257,9 @@ impl NamespaceResolver {
 				}
 				// Namespaces for XML 1.0
 				// Namespace constraint: Prefix Declared
-				Err(Error::NotNamespaceWellFormed(
-					NWFError::UndeclaredNamespacePrefix(errctx::ERRCTX_UNKNOWN),
-				))
+				Err(Error::Xml(XmlError::UndeclaredNamespacePrefix(
+					errctx::ERRCTX_UNKNOWN,
+				)))
 			}
 		}
 	}
@@ -298,9 +296,7 @@ impl NamespaceResolver {
 				// Namespaces in XML 1.0
 				// Namespace constraint: Attributes Unique
 				// We cannot distinguish between the two violations at this point anymore, and the difference is in most cases irrelevant, so we don't.
-				Entry::Occupied(_) => {
-					return Err(Error::NotWellFormed(WFError::DuplicateAttribute))
-				}
+				Entry::Occupied(_) => return Err(Error::Xml(XmlError::DuplicateAttribute)),
 				Entry::Vacant(e) => e.insert(value),
 			};
 		}
@@ -354,9 +350,9 @@ impl NamespaceResolver {
 				self.namespace_stack.pop();
 				Ok(Some(ResolvedEvent::EndElement(em)))
 			}
-			RawEvent::XMLDeclaration(em, v) => {
+			RawEvent::XmlDeclaration(em, v) => {
 				self.event_length_accum = 0;
-				Ok(Some(ResolvedEvent::XMLDeclaration(em, v)))
+				Ok(Some(ResolvedEvent::XmlDeclaration(em, v)))
 			}
 			RawEvent::Text(em, v) => {
 				self.event_length_accum = 0;
@@ -427,16 +423,16 @@ mod tests {
 
 	#[test]
 	fn namespace_resolver_passes_xml_decl() {
-		let (evs, r) = resolve_all(vec![RawEvent::XMLDeclaration(
+		let (evs, r) = resolve_all(vec![RawEvent::XmlDeclaration(
 			EventMetrics { len: 2342 },
-			XMLVersion::V1_0,
+			XmlVersion::V1_0,
 		)]);
 		r.unwrap();
 		let mut iter = evs.iter();
 		match iter.next().unwrap() {
-			ResolvedEvent::XMLDeclaration(em, v) => {
+			ResolvedEvent::XmlDeclaration(em, v) => {
 				assert_eq!(em.len(), 2342);
-				assert_eq!(*v, XMLVersion::V1_0);
+				assert_eq!(*v, XmlVersion::V1_0);
 			}
 			other => panic!("unexpected event: {:?}", other),
 		}
@@ -578,7 +574,7 @@ mod tests {
 			RawEvent::ElementFoot(EventMetrics { len: 6 }),
 		]);
 		match r {
-			Err(Error::NotWellFormed(WFError::DuplicateAttribute)) => (),
+			Err(Error::Xml(XmlError::DuplicateAttribute)) => (),
 			other => panic!("unexpected result: {:?}", other),
 		}
 		let mut iter = evs.iter();
@@ -619,14 +615,14 @@ mod tests {
 		{
 			let mut iter = pevs_invalid.iter();
 			match nsr.next(|| Ok(iter.next().cloned())) {
-				Err(Error::NotWellFormed(WFError::DuplicateAttribute)) => (),
+				Err(Error::Xml(XmlError::DuplicateAttribute)) => (),
 				other => panic!("unexpected result: {:?}", other),
 			}
 		}
 		{
 			let mut iter = pevs_valid.iter();
 			match nsr.next(|| Ok(iter.next().cloned())) {
-				Err(Error::NotWellFormed(WFError::DuplicateAttribute)) => (),
+				Err(Error::Xml(XmlError::DuplicateAttribute)) => (),
 				other => panic!("unexpected result: {:?}", other),
 			}
 		}
@@ -883,9 +879,7 @@ mod tests {
 			other => panic!("unexpected event: {:?}", other),
 		}
 		match r {
-			Err(Error::NotNamespaceWellFormed(NWFError::UndeclaredNamespacePrefix(
-				errctx::ERRCTX_NAME,
-			))) => (),
+			Err(Error::Xml(XmlError::UndeclaredNamespacePrefix(errctx::ERRCTX_NAME))) => (),
 			other => panic!("unexpected result: {:?}", other),
 		}
 		match iter.next() {
@@ -937,9 +931,7 @@ mod tests {
 			other => panic!("unexpected event: {:?}", other),
 		}
 		match r {
-			Err(Error::NotNamespaceWellFormed(NWFError::UndeclaredNamespacePrefix(
-				errctx::ERRCTX_ATTNAME,
-			))) => (),
+			Err(Error::Xml(XmlError::UndeclaredNamespacePrefix(errctx::ERRCTX_ATTNAME))) => (),
 			other => panic!("unexpected result: {:?}", other),
 		}
 		match iter.next() {
@@ -988,7 +980,7 @@ mod tests {
 			other => panic!("unexpected event: {:?}", other),
 		}
 		match r {
-			Err(Error::NotWellFormed(WFError::DuplicateAttribute)) => (),
+			Err(Error::Xml(XmlError::DuplicateAttribute)) => (),
 			other => panic!("unexpected result: {:?}", other),
 		}
 		match iter.next() {
@@ -1016,7 +1008,7 @@ mod tests {
 		]);
 		let mut iter = evs.iter();
 		match r {
-			Err(Error::NotWellFormed(WFError::DuplicateAttribute)) => (),
+			Err(Error::Xml(XmlError::DuplicateAttribute)) => (),
 			other => panic!("unexpected result: {:?}", other),
 		}
 		match iter.next() {
