@@ -12,7 +12,7 @@ use super::common::*;
 
 /// Pair of an optional namespace prefix and a localpart, commonly used in
 /// element and attribute names.
-pub type RawQName = (Option<NCName>, NCName);
+pub type RawQName = (Option<NcName>, NcName);
 
 /**
 # Logical XML document parts
@@ -31,7 +31,7 @@ number of bytes from the input stream used to generate the event.
 
 A well-formed XML document will generate the following sequence of events:
 
-1. Zero or one [`Self::XMLDeclaration`]
+1. Zero or one [`Self::XmlDeclaration`]
 2. One *element sequence*
 
 An *element sequence* consists of:
@@ -48,14 +48,14 @@ pub enum RawEvent {
 	///
 	/// As the `encoding` and `standalone` flag are forced to be `utf-8` and
 	/// `yes` respectively (or absent), those values are not emitted.
-	XMLDeclaration(
+	XmlDeclaration(
 		/// Number of bytes contributing to this event.
 		///
 		/// This includes all bytes from the opening `<?` until and including
 		/// the closing `?>`.
 		EventMetrics,
 		/// XML version number
-		XMLVersion,
+		XmlVersion,
 	),
 
 	/// Start of an XML element header
@@ -148,7 +148,7 @@ impl RawEvent {
 	/// Return the [`EventMetrics`] of the event
 	pub fn metrics(&self) -> &EventMetrics {
 		match self {
-			Self::XMLDeclaration(m, ..) => &m,
+			Self::XmlDeclaration(m, ..) => &m,
 			Self::ElementHeadOpen(m, ..) => &m,
 			Self::Attribute(m, ..) => &m,
 			Self::ElementHeadClose(m, ..) => &m,
@@ -193,7 +193,7 @@ enum State {
 	Initial,
 	Decl {
 		substate: DeclSt,
-		version: Option<XMLVersion>,
+		version: Option<XmlVersion>,
 	},
 	Document(DocSt),
 	End,
@@ -367,7 +367,7 @@ impl RawParser {
 				self.event_length = 0;
 				Ok(State::Document(DocSt::Element(ElementSt::AttrName)))
 			}
-			Some(tok) => Err(Error::NotWellFormed(WFError::UnexpectedToken(
+			Some(tok) => Err(Error::Xml(XmlError::UnexpectedToken(
 				ERRCTX_DOCBEGIN,
 				tok.name(),
 				Some(&[Token::NAME_ELEMENTHEADSTART, Token::NAME_XMLDECLSTART]),
@@ -382,7 +382,7 @@ impl RawParser {
 	fn parse_decl<'r, R: TokenRead>(
 		&mut self,
 		state: DeclSt,
-		version: Option<XMLVersion>,
+		version: Option<XmlVersion>,
 		r: &'r mut R,
 	) -> Result<State> {
 		match self.read_token(r)? {
@@ -396,7 +396,7 @@ impl RawParser {
 								version: version,
 							})
 						} else {
-							Err(Error::NotWellFormed(WFError::InvalidSyntax(
+							Err(Error::Xml(XmlError::InvalidSyntax(
 								"'<?xml' must be followed by version attribute",
 							)))
 						}
@@ -408,7 +408,7 @@ impl RawParser {
 								version: version,
 							})
 						} else {
-							Err(Error::NotWellFormed(WFError::InvalidSyntax("'version' attribute must be followed by '?>' or 'encoding' attribute")))
+							Err(Error::Xml(XmlError::InvalidSyntax("'version' attribute must be followed by '?>' or 'encoding' attribute")))
 						}
 					}
 					DeclSt::StandaloneName => {
@@ -418,10 +418,10 @@ impl RawParser {
 								version: version,
 							})
 						} else {
-							Err(Error::NotWellFormed(WFError::InvalidSyntax("'encoding' attribute must be followed by '?>' or 'standalone' attribute")))
+							Err(Error::Xml(XmlError::InvalidSyntax("'encoding' attribute must be followed by '?>' or 'standalone' attribute")))
 						}
 					}
-					_ => Err(Error::NotWellFormed(WFError::UnexpectedToken(
+					_ => Err(Error::Xml(XmlError::UnexpectedToken(
 						ERRCTX_XML_DECL,
 						Token::NAME_NAME,
 						None, // TODO: add expected tokens here
@@ -433,7 +433,7 @@ impl RawParser {
 					DeclSt::VersionEq => Ok(DeclSt::VersionValue),
 					DeclSt::EncodingEq => Ok(DeclSt::EncodingValue),
 					DeclSt::StandaloneEq => Ok(DeclSt::StandaloneValue),
-					_ => Err(Error::NotWellFormed(WFError::UnexpectedToken(
+					_ => Err(Error::Xml(XmlError::UnexpectedToken(
 						ERRCTX_XML_DECL,
 						Token::NAME_EQ,
 						None,
@@ -446,7 +446,7 @@ impl RawParser {
 					if v == "1.0" {
 						Ok(State::Decl {
 							substate: DeclSt::EncodingName,
-							version: Some(XMLVersion::V1_0),
+							version: Some(XmlVersion::V1_0),
 						})
 					} else {
 						Err(Error::RestrictedXml("only XML version 1.0 is allowed"))
@@ -474,7 +474,7 @@ impl RawParser {
 						))
 					}
 				}
-				_ => Err(Error::NotWellFormed(WFError::UnexpectedToken(
+				_ => Err(Error::Xml(XmlError::UnexpectedToken(
 					ERRCTX_XML_DECL,
 					Token::NAME_ATTRIBUTEVALUE,
 					None,
@@ -482,17 +482,17 @@ impl RawParser {
 			},
 			Some(Token::XMLDeclEnd(_)) => match state {
 				DeclSt::EncodingName | DeclSt::StandaloneName | DeclSt::Close => {
-					let ev = RawEvent::XMLDeclaration(self.finish_event(), version.unwrap());
+					let ev = RawEvent::XmlDeclaration(self.finish_event(), version.unwrap());
 					self.emit_event(ev);
 					Ok(State::Document(DocSt::Element(ElementSt::Expected)))
 				}
-				_ => Err(Error::NotWellFormed(WFError::UnexpectedToken(
+				_ => Err(Error::Xml(XmlError::UnexpectedToken(
 					ERRCTX_XML_DECL,
 					Token::NAME_XMLDECLEND,
 					None,
 				))),
 			},
-			Some(other) => Err(Error::NotWellFormed(WFError::UnexpectedToken(
+			Some(other) => Err(Error::Xml(XmlError::UnexpectedToken(
 				ERRCTX_XML_DECL,
 				other.name(),
 				None,
@@ -509,29 +509,23 @@ impl RawParser {
 				// Namespace constraint: Reserved Prefixes and Namespace Names
 				if localpart == "xml" {
 					if val != XMLNS_XML {
-						return Err(Error::NotNamespaceWellFormed(
-							NWFError::ReservedNamespacePrefix,
-						));
+						return Err(Error::Xml(XmlError::ReservedNamespacePrefix));
 					}
 				} else {
 					if val == XMLNS_XML {
-						return Err(Error::NotNamespaceWellFormed(
-							NWFError::ReservedNamespaceName,
-						));
+						return Err(Error::Xml(XmlError::ReservedNamespaceName));
 					}
 				}
 				// Namespaces for XML 1.0
 				// Namespace constraint: No Prefix Undeclaring
 				if val.len() == 0 {
-					return Err(Error::NotNamespaceWellFormed(NWFError::EmptyNamespaceUri));
+					return Err(Error::Xml(XmlError::EmptyNamespaceUri));
 				}
 			}
 		} else if localpart == "xmlns" && val == XMLNS_XML {
 			// Namespaces for XML 1.0
 			// Namespace constraint: Reserved Prefixes and Namespace Names
-			return Err(Error::NotNamespaceWellFormed(
-				NWFError::ReservedNamespaceName,
-			));
+			return Err(Error::Xml(XmlError::ReservedNamespaceName));
 		}
 		Ok(RawEvent::Attribute(
 			self.finish_event(),
@@ -569,7 +563,7 @@ impl RawParser {
 					self.emit_event(RawEvent::ElementHeadClose(em));
 					Ok(State::Document(DocSt::CData))
 				}
-				_ => Err(Error::NotWellFormed(WFError::UnexpectedToken(
+				_ => Err(Error::Xml(XmlError::UnexpectedToken(
 					ERRCTX_ELEMENT,
 					Token::NAME_ELEMENTHEADCLOSE,
 					None,
@@ -584,7 +578,7 @@ impl RawParser {
 					self.emit_event(RawEvent::ElementHeadClose(em));
 					Ok(self.pop_element(self.fixed_event(0))?)
 				}
-				_ => Err(Error::NotWellFormed(WFError::UnexpectedToken(
+				_ => Err(Error::Xml(XmlError::UnexpectedToken(
 					ERRCTX_ELEMENT,
 					Token::NAME_ELEMENTHEADCLOSE,
 					None,
@@ -598,15 +592,13 @@ impl RawParser {
 					let (prefix, localname) = add_context(name.split_name(), ERRCTX_ATTNAME)?;
 					if let Some(prefix) = prefix.as_ref() {
 						if prefix == "xmlns" && localname == "xmlns" {
-							return Err(Error::NotNamespaceWellFormed(
-								NWFError::ReservedNamespacePrefix,
-							));
+							return Err(Error::Xml(XmlError::ReservedNamespacePrefix));
 						}
 					}
 					self.attribute_scratchpad = Some((prefix, localname));
 					Ok(State::Document(DocSt::Element(ElementSt::AttrEq)))
 				}
-				_ => Err(Error::NotWellFormed(WFError::UnexpectedToken(
+				_ => Err(Error::Xml(XmlError::UnexpectedToken(
 					ERRCTX_ELEMENT,
 					Token::NAME_NAME,
 					None,
@@ -614,7 +606,7 @@ impl RawParser {
 			},
 			Some(Token::Eq(_)) => match state {
 				ElementSt::AttrEq => Ok(State::Document(DocSt::Element(ElementSt::AttrValue))),
-				_ => Err(Error::NotWellFormed(WFError::UnexpectedToken(
+				_ => Err(Error::Xml(XmlError::UnexpectedToken(
 					ERRCTX_ELEMENT,
 					Token::NAME_EQ,
 					None,
@@ -631,13 +623,13 @@ impl RawParser {
 					self.event_length = 0;
 					Ok(State::Document(DocSt::Element(ElementSt::AttrName)))
 				}
-				_ => Err(Error::NotWellFormed(WFError::UnexpectedToken(
+				_ => Err(Error::Xml(XmlError::UnexpectedToken(
 					ERRCTX_ELEMENT,
 					Token::NAME_EQ,
 					None,
 				))),
 			},
-			Some(tok) => Err(Error::NotWellFormed(WFError::UnexpectedToken(
+			Some(tok) => Err(Error::Xml(XmlError::UnexpectedToken(
 				ERRCTX_ELEMENT,
 				tok.name(),
 				None,
@@ -672,12 +664,12 @@ impl RawParser {
 				Some(Token::ElementFootStart(tm, name)) => {
 					self.start_event(&tm);
 					if self.element_stack[self.element_stack.len() - 1] != name {
-						Err(Error::NotWellFormed(WFError::ElementMismatch))
+						Err(Error::Xml(XmlError::ElementMismatch))
 					} else {
 						Ok(State::Document(DocSt::ElementFoot))
 					}
 				}
-				Some(tok) => Err(Error::NotWellFormed(WFError::UnexpectedToken(
+				Some(tok) => Err(Error::Xml(XmlError::UnexpectedToken(
 					ERRCTX_TEXT,
 					tok.name(),
 					Some(&[
@@ -693,7 +685,7 @@ impl RawParser {
 					let ev = self.finish_event();
 					self.pop_element(ev)
 				}
-				Some(other) => Err(Error::NotWellFormed(WFError::UnexpectedToken(
+				Some(other) => Err(Error::Xml(XmlError::UnexpectedToken(
 					ERRCTX_ELEMENT_FOOT,
 					other.name(),
 					Some(&[Token::NAME_ELEMENTHFEND]),
@@ -728,7 +720,7 @@ impl Parse for RawParser {
 					{
 						Ok(State::End)
 					}
-					Some(tok) => Err(Error::NotWellFormed(WFError::UnexpectedToken(
+					Some(tok) => Err(Error::Xml(XmlError::UnexpectedToken(
 						ERRCTX_DOCEND,
 						tok.name(),
 						Some(&["end-of-file"]),
@@ -881,7 +873,7 @@ mod tests {
 		]);
 		let mut iter = evs.iter();
 		match iter.next().unwrap() {
-			RawEvent::XMLDeclaration(em, XMLVersion::V1_0) => {
+			RawEvent::XmlDeclaration(em, XmlVersion::V1_0) => {
 				assert_eq!(em.len(), 7);
 			}
 			other => panic!("unexpected event: {:?}", other),
@@ -889,7 +881,7 @@ mod tests {
 		assert!(iter.next().is_none());
 		assert!(matches!(
 			r.err().unwrap(),
-			Error::NotWellFormed(WFError::InvalidEof(ERRCTX_DOCBEGIN))
+			Error::Xml(XmlError::InvalidEof(ERRCTX_DOCBEGIN))
 		));
 	}
 
@@ -930,7 +922,7 @@ mod tests {
 		loop {
 			match parser.parse(&mut reader) {
 				Err(Error::IO(ioerr)) if ioerr.kind() == io::ErrorKind::WouldBlock => continue,
-				Err(Error::NotWellFormed(WFError::InvalidEof(ERRCTX_DOCBEGIN))) => break,
+				Err(Error::Xml(XmlError::InvalidEof(ERRCTX_DOCBEGIN))) => break,
 				Err(other) => panic!("unexpected error: {:?}", other),
 				Ok(Some(ev)) => evs.push(ev),
 				Ok(None) => panic!("unexpected eof: {:?}", parser),
@@ -938,7 +930,7 @@ mod tests {
 		}
 		assert!(matches!(
 			&evs[0],
-			RawEvent::XMLDeclaration(EventMetrics { len: 0 }, XMLVersion::V1_0)
+			RawEvent::XmlDeclaration(EventMetrics { len: 0 }, XmlVersion::V1_0)
 		));
 		assert_eq!(evs.len(), 1);
 	}
@@ -958,7 +950,7 @@ mod tests {
 		let r = parser.parse(&mut reader);
 		assert!(matches!(
 			r.unwrap().unwrap(),
-			RawEvent::XMLDeclaration(EventMetrics { len: 0 }, XMLVersion::V1_0)
+			RawEvent::XmlDeclaration(EventMetrics { len: 0 }, XmlVersion::V1_0)
 		));
 	}
 
@@ -975,7 +967,7 @@ mod tests {
 		]);
 		r.unwrap();
 		match evs.remove(0) {
-			RawEvent::XMLDeclaration(_, XMLVersion::V1_0) => (),
+			RawEvent::XmlDeclaration(_, XmlVersion::V1_0) => (),
 			other => panic!("unexpected event: {:?}", other),
 		}
 		match evs.remove(0) {
@@ -1209,7 +1201,7 @@ mod tests {
 			ev => panic!("unexpected event: {:?}", ev),
 		}
 		match r {
-			Err(Error::NotNamespaceWellFormed(NWFError::ReservedNamespacePrefix)) => (),
+			Err(Error::Xml(XmlError::ReservedNamespacePrefix)) => (),
 			other => panic!("unexpected result: {:?}", other),
 		}
 		assert_eq!(evs.len(), 0);
@@ -1271,7 +1263,7 @@ mod tests {
 			ev => panic!("unexpected event: {:?}", ev),
 		}
 		match r {
-			Err(Error::NotNamespaceWellFormed(NWFError::ReservedNamespacePrefix)) => (),
+			Err(Error::Xml(XmlError::ReservedNamespacePrefix)) => (),
 			other => panic!("unexpected result: {:?}", other),
 		}
 		assert_eq!(evs.len(), 0);
@@ -1295,7 +1287,7 @@ mod tests {
 			ev => panic!("unexpected event: {:?}", ev),
 		}
 		match r {
-			Err(Error::NotNamespaceWellFormed(NWFError::ReservedNamespaceName)) => (),
+			Err(Error::Xml(XmlError::ReservedNamespaceName)) => (),
 			other => panic!("unexpected result: {:?}", other),
 		}
 		assert_eq!(evs.len(), 0);
@@ -1319,7 +1311,7 @@ mod tests {
 			ev => panic!("unexpected event: {:?}", ev),
 		}
 		match r {
-			Err(Error::NotNamespaceWellFormed(NWFError::ReservedNamespaceName)) => (),
+			Err(Error::Xml(XmlError::ReservedNamespaceName)) => (),
 			other => panic!("unexpected result: {:?}", other),
 		}
 		assert_eq!(evs.len(), 0);
@@ -1478,7 +1470,7 @@ mod tests {
 			Token::ElementHFEnd(DM),
 		]);
 		match r {
-			Err(Error::NotWellFormed(WFError::ElementMismatch)) => (),
+			Err(Error::Xml(XmlError::ElementMismatch)) => (),
 			other => panic!("unexpected result: {:?}", other),
 		}
 		let mut iter = evs.iter();
@@ -1713,12 +1705,12 @@ mod tests {
 		}
 		let r = parser.parse(&mut reader);
 		match r {
-			Err(Error::NotNamespaceWellFormed(NWFError::ReservedNamespacePrefix)) => (),
+			Err(Error::Xml(XmlError::ReservedNamespacePrefix)) => (),
 			other => panic!("unexpected result: {:?}", other),
 		}
 		let r = parser.parse(&mut reader);
 		match r {
-			Err(Error::NotNamespaceWellFormed(NWFError::ReservedNamespacePrefix)) => (),
+			Err(Error::Xml(XmlError::ReservedNamespacePrefix)) => (),
 			other => panic!("unexpected result: {:?}", other),
 		}
 	}
@@ -1736,7 +1728,7 @@ mod tests {
 		];
 		let err = parse_err(toks).unwrap();
 		match err {
-			Error::NotNamespaceWellFormed(NWFError::EmptyNamespaceUri) => (),
+			Error::Xml(XmlError::EmptyNamespaceUri) => (),
 			other => panic!("unexpected error: {:?}", other),
 		}
 	}
@@ -1790,7 +1782,7 @@ mod tests {
 			other => panic!("unexpected event: {:?}", other),
 		}
 		match r {
-			Err(Error::NotWellFormed(WFError::UnexpectedToken(_, _, _))) => (),
+			Err(Error::Xml(XmlError::UnexpectedToken(_, _, _))) => (),
 			other => panic!("unexpected result: {:?}", other),
 		}
 	}
@@ -1826,7 +1818,7 @@ mod tests {
 			other => panic!("unexpected event: {:?}", other),
 		}
 		match r {
-			Err(Error::NotWellFormed(WFError::UnexpectedToken(_, _, _))) => (),
+			Err(Error::Xml(XmlError::UnexpectedToken(_, _, _))) => (),
 			other => panic!("unexpected result: {:?}", other),
 		}
 	}
@@ -1876,7 +1868,7 @@ mod tests {
 			Token::ElementHFEnd(DM),
 		]);
 		match err {
-			Some(Error::NotWellFormed(WFError::UnexpectedToken(..))) => (),
+			Some(Error::Xml(XmlError::UnexpectedToken(..))) => (),
 			other => panic!("unexpected error: {:?}", other),
 		}
 	}
