@@ -22,7 +22,7 @@ fn feedparser_can_read_xml_document() {
 	{
 		let mut iter = out.iter();
 		match iter.next().unwrap() {
-			ResolvedEvent::XMLDeclaration(em, XMLVersion::V1_0) => {
+			ResolvedEvent::XmlDeclaration(em, XmlVersion::V1_0) => {
 				assert_eq!(em.len(), 21);
 			}
 			other => panic!("unexpected event: {:?}", other),
@@ -38,11 +38,11 @@ fn feedparser_can_read_xml_document() {
 				assert_eq!(localname, "root");
 				assert_eq!(attrs.len(), 2);
 				assert_eq!(
-					attrs.get(&(None, NCName::try_from("a").unwrap())).unwrap(),
+					attrs.get(&(None, NcName::try_from("a").unwrap())).unwrap(),
 					"foo"
 				);
 				assert_eq!(
-					attrs.get(&(None, NCName::try_from("b").unwrap())).unwrap(),
+					attrs.get(&(None, NcName::try_from("b").unwrap())).unwrap(),
 					"bar"
 				);
 			}
@@ -108,7 +108,7 @@ fn feedparser_can_handle_chunked_input() {
 	{
 		let mut iter = out.iter();
 		match iter.next().unwrap() {
-			ResolvedEvent::XMLDeclaration(em, XMLVersion::V1_0) => {
+			ResolvedEvent::XmlDeclaration(em, XmlVersion::V1_0) => {
 				assert_eq!(em.len(), 21);
 			}
 			other => panic!("unexpected event: {:?}", other),
@@ -123,11 +123,11 @@ fn feedparser_can_handle_chunked_input() {
 				assert_eq!(localname, "root");
 				assert_eq!(attrs.len(), 2);
 				assert_eq!(
-					attrs.get(&(None, NCName::try_from("a").unwrap())).unwrap(),
+					attrs.get(&(None, NcName::try_from("a").unwrap())).unwrap(),
 					"foo"
 				);
 				assert_eq!(
-					attrs.get(&(None, NCName::try_from("b").unwrap())).unwrap(),
+					attrs.get(&(None, NcName::try_from("b").unwrap())).unwrap(),
 					"bar"
 				);
 			}
@@ -188,7 +188,7 @@ fn pullparser_can_read_xml_document() {
 	{
 		let mut iter = out.iter();
 		match iter.next().unwrap() {
-			ResolvedEvent::XMLDeclaration(em, XMLVersion::V1_0) => {
+			ResolvedEvent::XmlDeclaration(em, XmlVersion::V1_0) => {
 				assert_eq!(em.len(), 21);
 			}
 			other => panic!("unexpected event: {:?}", other),
@@ -204,11 +204,11 @@ fn pullparser_can_read_xml_document() {
 				assert_eq!(localname, "root");
 				assert_eq!(attrs.len(), 2);
 				assert_eq!(
-					attrs.get(&(None, NCName::try_from("a").unwrap())).unwrap(),
+					attrs.get(&(None, NcName::try_from("a").unwrap())).unwrap(),
 					"foo"
 				);
 				assert_eq!(
-					attrs.get(&(None, NCName::try_from("b").unwrap())).unwrap(),
+					attrs.get(&(None, NcName::try_from("b").unwrap())).unwrap(),
 					"bar"
 				);
 			}
@@ -279,7 +279,7 @@ async fn asyncparser_can_read_xml_document() {
 	{
 		let mut iter = out.iter();
 		match iter.next().unwrap() {
-			ResolvedEvent::XMLDeclaration(em, XMLVersion::V1_0) => {
+			ResolvedEvent::XmlDeclaration(em, XmlVersion::V1_0) => {
 				assert_eq!(em.len(), 21);
 			}
 			other => panic!("unexpected event: {:?}", other),
@@ -295,11 +295,11 @@ async fn asyncparser_can_read_xml_document() {
 				assert_eq!(localname, "root");
 				assert_eq!(attrs.len(), 2);
 				assert_eq!(
-					attrs.get(&(None, NCName::try_from("a").unwrap())).unwrap(),
+					attrs.get(&(None, NcName::try_from("a").unwrap())).unwrap(),
 					"foo"
 				);
 				assert_eq!(
-					attrs.get(&(None, NCName::try_from("b").unwrap())).unwrap(),
+					attrs.get(&(None, NcName::try_from("b").unwrap())).unwrap(),
 					"bar"
 				);
 			}
@@ -358,7 +358,7 @@ async fn asyncparser_can_handle_chunked_input() {
 	{
 		let mut iter = out.iter();
 		match iter.next().unwrap() {
-			ResolvedEvent::XMLDeclaration(em, XMLVersion::V1_0) => {
+			ResolvedEvent::XmlDeclaration(em, XmlVersion::V1_0) => {
 				assert_eq!(em.len(), 21);
 			}
 			other => panic!("unexpected event: {:?}", other),
@@ -374,11 +374,11 @@ async fn asyncparser_can_handle_chunked_input() {
 				assert_eq!(localname, "root");
 				assert_eq!(attrs.len(), 2);
 				assert_eq!(
-					attrs.get(&(None, NCName::try_from("a").unwrap())).unwrap(),
+					attrs.get(&(None, NcName::try_from("a").unwrap())).unwrap(),
 					"foo"
 				);
 				assert_eq!(
-					attrs.get(&(None, NCName::try_from("b").unwrap())).unwrap(),
+					attrs.get(&(None, NcName::try_from("b").unwrap())).unwrap(),
 					"bar"
 				);
 			}
