@@ -6,15 +6,16 @@ use std::collections::HashMap;
 use std::collections::HashSet;
 use std::convert::TryInto;
 use std::fmt;
+use std::sync::Arc;
 
 use bytes::{BufMut, BytesMut};
 
-use crate::parser::{NamespaceName, RcPtr, ResolvedEvent, XMLVersion, XMLNS_XML, XMLNS_XMLNS};
-use crate::strings::{CData, CDataStr, NCName, NCNameStr, Name};
+use crate::parser::{NamespaceName, RcPtr, ResolvedEvent, XmlVersion, XMLNS_XML, XMLNS_XMLNS};
+use crate::strings::{CData, CDataStr, Name, NcName, NcNameStr};
 
-static XML_DECL: &'static [u8] = b"<?xml version='1.0' encoding='utf-8'?>\n";
-pub const PREFIX_XML: &'static NCNameStr = unsafe { std::mem::transmute("xml") };
-pub const PREFIX_XMLNS: &'static NCNameStr = unsafe { std::mem::transmute("xmlns") };
+const XML_DECL: &'static [u8] = b"<?xml version='1.0' encoding='utf-8'?>\n";
+pub const PREFIX_XML: &'static NcNameStr = unsafe { std::mem::transmute("xml") };
+pub const PREFIX_XMLNS: &'static NcNameStr = unsafe { std::mem::transmute("xmlns") };
 
 const CDATA_SPECIALS: &'static [u8] = &[b'<', b'>', b'&', b'\r'];
 
@@ -55,14 +56,14 @@ fn escape<'a, B: BufMut>(out: &'a mut B, data: &'a [u8], specials: &'static [u8]
 ///   [`ResolvedEvent`]: crate::parser::ResolvedEvent
 pub enum Item<'x> {
 	/// XML declaration
-	XMLDeclaration(XMLVersion),
+	XmlDeclaration(XmlVersion),
 
 	/// Start of an element header
 	ElementHeadStart(
 		/// Namespace URI or None, for unnamespaced elements
 		Option<RcPtr<CData>>,
 		/// Local name of the attribute
-		&'x NCNameStr,
+		&'x NcNameStr,
 	),
 
 	/// An attribute key/value pair
@@ -70,7 +71,7 @@ pub enum Item<'x> {
 		/// Namespace URI or None, for unnamespaced attributes
 		Option<RcPtr<CData>>,
 		/// Local name of the attribute
-		&'x NCNameStr,
+		&'x NcNameStr,
 		/// Value of the attribute
 		&'x CDataStr,
 	),
@@ -124,7 +125,7 @@ pub trait TrackNamespace {
 	///
 	/// Calling this twice between two calls to `push` with the same `prefix`
 	/// is a programming error and causes a panic.
-	fn declare_fixed(&mut self, prefix: Option<&NCNameStr>, name: Option<NamespaceName>) -> bool;
+	fn declare_fixed(&mut self, prefix: Option<&NcNameStr>, name: Option<NamespaceName>) -> bool;
 
 	/// Declare a namespace URI with an auto-generated prefix or by using the
 	/// default namespace.
@@ -138,7 +139,7 @@ pub trait TrackNamespace {
 	///
 	/// This may return a non-auto-generated prefix if the namespace URI is
 	/// already declared on this or a parent element.
-	fn declare_auto(&mut self, name: Option<NamespaceName>) -> (bool, Option<&NCNameStr>);
+	fn declare_auto(&mut self, name: Option<NamespaceName>) -> (bool, Option<&NcNameStr>);
 
 	/// Declare a namespace URI with an auto-generated prefix.
 	///
@@ -153,26 +154,45 @@ pub trait TrackNamespace {
 	/// already declared on this or a parent element. If the URI is already
 	/// used for the default namespace, this function will nontheless return
 	/// a prefix.
-	fn declare_with_auto_prefix(&mut self, name: Option<NamespaceName>) -> (bool, &NCNameStr);
+	fn declare_with_auto_prefix(&mut self, name: Option<NamespaceName>) -> (bool, &NcNameStr);
 
 	/// Get the prefix for a given URI, which may be empty if the namespace
 	/// with that URI is defined as the default namespace.
 	fn get_prefix_or_default(
 		&self,
 		name: Option<NamespaceName>,
-	) -> Result<Option<&NCNameStr>, PrefixError>;
+	) -> Result<Option<&NcNameStr>, PrefixError>;
 
 	/// Get the prefix for a given URI.
 	///
 	/// This returns an error if the given URI is declared as default
 	/// namespace and there is no matching prefix.
-	fn get_prefix(&self, name: Option<NamespaceName>) -> Result<&NCNameStr, PrefixError>;
+	fn get_prefix(&self, name: Option<NamespaceName>) -> Result<&NcNameStr, PrefixError>;
 
 	/// Complete an element declaration.
 	fn push(&mut self);
 
 	/// Signal end of element to undeclare nested namespace declarations.
 	fn pop(&mut self);
+
+	/// Return the newly declared default namespace, if any.
+	///
+	/// This returns `None` if and only if no default namespace has been
+	/// declared yet for the upcoming element.
+	///
+	/// `Some(None)` is returned if the empty default namespace has been
+	/// declared, effectively resetting the default namespace to the initial
+	/// state in an XML document.
+	///
+	/// *Note:* In the root element, `None` will be returned even if the
+	/// default namespace has been explicitly set to `None`, as that is the
+	/// default.
+	fn new_default_declaration(&self) -> Option<Option<&NamespaceName>>;
+
+	/// Return an iterator over the the newly declared prefixes.
+	fn new_prefix_declarations<'x>(
+		&'x self,
+	) -> Box<dyn Iterator<Item = (&Option<NamespaceName>, &NcNameStr)> + 'x>;
 }
 
 /// Simple namespace tracker.
@@ -188,18 +208,19 @@ pub trait TrackNamespace {
 ///
 /// One exception is that prefixed namespaces declared on the root element
 /// will actually be made available on all child elements.
+#[derive(Debug)]
 pub struct SimpleNamespaces {
 	// persistent state
-	global_ns: HashMap<Option<NamespaceName>, NCName>,
-	global_ns_rev: HashSet<NCName>,
+	global_ns: HashMap<Option<NamespaceName>, NcName>,
+	global_ns_rev: HashSet<NcName>,
 	global_ns_ctr: usize,
 	default_ns_stack: Vec<Option<NamespaceName>>,
 
 	// temporary per-element state
 	next_default_ns: Option<Option<NamespaceName>>,
 	temp_ns_ctr: usize,
-	temp_ns: HashMap<Option<NamespaceName>, NCName>,
-	temp_ns_rev: HashSet<NCName>,
+	temp_ns: HashMap<Option<NamespaceName>, NcName>,
+	temp_ns_rev: HashSet<NcName>,
 }
 
 impl SimpleNamespaces {
@@ -216,10 +237,45 @@ impl SimpleNamespaces {
 			temp_ns_rev: HashSet::new(),
 		}
 	}
+
+	/// Look up the namespace URI for a given prefix
+	///
+	/// *Note:* This function is implemented as O(n) function because it
+	/// should rarely, if ever, be necessary to use. Speeding up runtime of
+	/// this function would increase memory cost at neglegible gain.
+	///
+	/// The namespace URIs for the `xml` and `xmlns` prefixes are always
+	/// returned, even if not explicitly declared.
+	pub fn lookup_prefix(&self, prefix: Option<&NcNameStr>) -> Result<NamespaceName, PrefixError> {
+		match prefix {
+			Some(prefix) if prefix == PREFIX_XML => Ok(RcPtr::new(XMLNS_XML.into())),
+			Some(prefix) if prefix == PREFIX_XMLNS => Ok(RcPtr::new(XMLNS_XMLNS.into())),
+			Some(prefix) => {
+				for (decl_uri, decl_prefix) in self.temp_ns.iter().chain(self.global_ns.iter()) {
+					if let Some(decl_uri) = decl_uri {
+						if decl_prefix == prefix {
+							return Ok(Arc::clone(decl_uri));
+						}
+					}
+				}
+				Err(PrefixError::Undeclared)
+			}
+			None => {
+				match self.next_default_ns.as_ref() {
+					Some(Some(uri)) => return Ok(Arc::clone(uri)),
+					_ => (),
+				};
+				match self.default_ns_stack.last() {
+					Some(Some(uri)) => Ok(Arc::clone(uri)),
+					_ => Err(PrefixError::Undeclared),
+				}
+			}
+		}
+	}
 }
 
 impl TrackNamespace for SimpleNamespaces {
-	fn declare_fixed(&mut self, prefix: Option<&NCNameStr>, name: Option<NamespaceName>) -> bool {
+	fn declare_fixed(&mut self, prefix: Option<&NcNameStr>, name: Option<NamespaceName>) -> bool {
 		match prefix.as_ref() {
 			Some(v) if *v == PREFIX_XML => {
 				if name.as_ref().map(|x| &***x) == Some(XMLNS_XML) {
@@ -271,32 +327,67 @@ impl TrackNamespace for SimpleNamespaces {
 		}
 	}
 
-	fn declare_auto(&mut self, name: Option<NamespaceName>) -> (bool, Option<&NCNameStr>) {
+	fn declare_auto(&mut self, name: Option<NamespaceName>) -> (bool, Option<&NcNameStr>) {
 		match name {
 			Some(v) if *v == XMLNS_XML => return (false, Some(PREFIX_XML)),
 			Some(v) if *v == XMLNS_XMLNS => return (false, Some(PREFIX_XMLNS)),
 			_ => (),
-		}
+		};
 
-		match self.next_default_ns.as_ref() {
-			Some(v) if *v == name => (false, None),
-			Some(v) => {
-				drop(v);
-				let (new, prefix) = self.declare_with_auto_prefix(name);
-				(new, Some(prefix))
-			}
-			None => {
-				self.next_default_ns = Some(name);
-				let new = match self.default_ns_stack.last() {
-					Some(v) => v != self.next_default_ns.as_ref().unwrap(),
-					None => self.next_default_ns.as_ref().unwrap().is_some(),
-				};
-				(new, None)
-			}
+		// XXX: due to a limitation in the borrowchecker <https://github.com/rust-lang/rfcs/blob/master/text/2094-nll.md#problem-case-3-conditional-control-flow-across-functions>, we have to re-implement the entirety of get_prefix_or_default here.
+
+		match self
+			.next_default_ns
+			.as_ref()
+			.or(self.default_ns_stack.last())
+		{
+			Some(default_name) if *default_name == name => return (false, None),
+			_ => (),
+		};
+
+		match self.temp_ns.entry(name.clone()) {
+			Entry::Occupied(o) => (false, Some(o.into_mut())),
+			Entry::Vacant(v_temp) => match self.global_ns.entry(name.clone()) {
+				Entry::Occupied(o) => (false, Some(o.into_mut())),
+				Entry::Vacant(_) => {
+					match self.next_default_ns.as_ref() {
+						// checked above already that it does not match
+						Some(_) => {
+							let ctr = self.temp_ns_ctr;
+							let temp_ns_prefix: NcName = format!("tns{}", ctr)
+								.try_into()
+								.expect("auto-generated prefix must always be valid");
+							if self.global_ns_rev.contains(&temp_ns_prefix) {
+								panic!(
+									"automatic prefix declaration conflicts with global prefix: {:?}",
+									temp_ns_prefix
+								)
+							}
+							if self.temp_ns_rev.contains(&temp_ns_prefix) {
+								panic!(
+									"automatic prefix declaration conflicts with local prefix: {:?}",
+									temp_ns_prefix
+								)
+							}
+							self.temp_ns_ctr += 1;
+							self.temp_ns_rev.insert(temp_ns_prefix.clone());
+							(true, Some(v_temp.insert(temp_ns_prefix)))
+						}
+						None => {
+							self.next_default_ns = Some(name);
+							let new = match self.default_ns_stack.last() {
+								Some(v) => v != self.next_default_ns.as_ref().unwrap(),
+								None => self.next_default_ns.as_ref().unwrap().is_some(),
+							};
+							(new, None)
+						}
+					}
+				}
+			},
 		}
 	}
 
-	fn declare_with_auto_prefix(&mut self, name: Option<NamespaceName>) -> (bool, &NCNameStr) {
+	fn declare_with_auto_prefix(&mut self, name: Option<NamespaceName>) -> (bool, &NcNameStr) {
 		match name {
 			Some(v) if *v == XMLNS_XML => return (false, PREFIX_XML),
 			Some(v) if *v == XMLNS_XMLNS => return (false, PREFIX_XMLNS),
@@ -307,7 +398,7 @@ impl TrackNamespace for SimpleNamespaces {
 			Entry::Occupied(o) => (false, o.into_mut()),
 			Entry::Vacant(v) => {
 				let ctr = self.temp_ns_ctr;
-				let temp_ns_prefix: NCName = format!("tns{}", ctr)
+				let temp_ns_prefix: NcName = format!("tns{}", ctr)
 					.try_into()
 					.expect("auto-generated prefix must always be valid");
 				if self.global_ns_rev.contains(&temp_ns_prefix) {
@@ -332,30 +423,23 @@ impl TrackNamespace for SimpleNamespaces {
 	fn get_prefix_or_default(
 		&self,
 		name: Option<NamespaceName>,
-	) -> Result<Option<&NCNameStr>, PrefixError> {
-		if let Some(next) = self.next_default_ns.as_ref() {
-			if *next == name {
-				return Ok(None);
-			}
+	) -> Result<Option<&NcNameStr>, PrefixError> {
+		match self
+			.next_default_ns
+			.as_ref()
+			.or(self.default_ns_stack.last())
+		{
+			Some(default_name) if *default_name == name => Ok(None),
+			_ => Ok(Some(self.get_prefix(name)?)),
 		}
-		if let Some(prev) = self.default_ns_stack.last() {
-			if *prev == name {
-				return Ok(None);
-			}
-		}
-		Ok(Some(self.get_prefix(name)?))
 	}
 
-	fn get_prefix(&self, name: Option<NamespaceName>) -> Result<&NCNameStr, PrefixError> {
-		match self.temp_ns.get(&name) {
-			Some(v) => return Ok(v),
-			None => (),
-		}
-		match self.global_ns.get(&name) {
-			Some(v) => return Ok(v),
-			None => (),
-		}
-		Err(PrefixError::Undeclared)
+	fn get_prefix(&self, name: Option<NamespaceName>) -> Result<&NcNameStr, PrefixError> {
+		self.temp_ns
+			.get(&name)
+			.or(self.global_ns.get(&name))
+			.map(|x| &**x)
+			.ok_or(PrefixError::Undeclared)
 	}
 
 	fn push(&mut self) {
@@ -383,13 +467,27 @@ impl TrackNamespace for SimpleNamespaces {
 	fn pop(&mut self) {
 		self.default_ns_stack.pop();
 	}
+
+	fn new_default_declaration(&self) -> Option<Option<&NamespaceName>> {
+		match self.next_default_ns.as_ref().map(|x| x.as_ref()) {
+			// if this is the root element, we do not expose the empty default namespace as declaration
+			Some(None) if self.default_ns_stack.len() == 0 => None,
+			other => other,
+		}
+	}
+
+	fn new_prefix_declarations<'x>(
+		&'x self,
+	) -> Box<dyn Iterator<Item = (&Option<NamespaceName>, &NcNameStr)> + 'x> {
+		Box::new(self.temp_ns.iter().map(|(k, v)| (k, &**v)))
+	}
 }
 
 #[derive(Debug, Clone, Copy, PartialEq, Eq)]
 pub enum EncodeError {
 	/// Emitted if an XML declaration is placed after the first element
 	/// started or if multiple XML declarations are placed.
-	MisplacedXMLDeclaration,
+	MisplacedXmlDeclaration,
 
 	/// Emitted if any content is placed after the end of the last element.
 	EndOfDocument,
@@ -413,7 +511,7 @@ pub enum EncodeError {
 impl fmt::Display for EncodeError {
 	fn fmt<'f>(&self, f: &'f mut fmt::Formatter) -> fmt::Result {
 		match self {
-			Self::MisplacedXMLDeclaration => f.write_str("misplaced XML declaration"),
+			Self::MisplacedXmlDeclaration => f.write_str("misplaced XML declaration"),
 			Self::ElementStartNotAllowed => {
 				f.write_str("element start not allowed inside element headers")
 			}
@@ -447,12 +545,12 @@ Encodes XML into buffers.
 Encoders are stateful. They can only be used to encode a single XML document and have then to be disposed.
 
 ```rust
-use rxml::{Encoder, Item, XMLVersion};
+use rxml::{Encoder, Item, XmlVersion};
 use bytes::BytesMut;
 
 let mut enc = Encoder::new();
 let mut buf = BytesMut::new();
-enc.encode(Item::XMLDeclaration(XMLVersion::V1_0), &mut buf);
+enc.encode(Item::XmlDeclaration(XmlVersion::V1_0), &mut buf);
 assert_eq!(&buf[..], b"<?xml version='1.0' encoding='utf-8'?>\n");
 ```
 */
@@ -492,7 +590,7 @@ impl<T: TrackNamespace> From<T> for Encoder<T> {
 
 impl<T: TrackNamespace> Encoder<T> {
 	fn encode_nsdecl<O: BufMut>(
-		prefix: Option<&NCNameStr>,
+		prefix: Option<&NcNameStr>,
 		nsuri: Option<&CDataStr>,
 		output: &mut O,
 	) {
@@ -512,6 +610,14 @@ impl<T: TrackNamespace> Encoder<T> {
 		output.put_u8(b'\'');
 	}
 
+	pub fn inner(&self) -> &T {
+		&self.ns
+	}
+
+	pub fn inner_mut(&mut self) -> &mut T {
+		&mut self.ns
+	}
+
 	/// Encode a single item into a buffer.
 	///
 	/// There is no requirement for the buffer to be the same for subsequent
@@ -524,22 +630,23 @@ impl<T: TrackNamespace> Encoder<T> {
 		}
 
 		match item {
-			Item::XMLDeclaration(XMLVersion::V1_0) => match self.state {
+			Item::XmlDeclaration(XmlVersion::V1_0) => match self.state {
 				EncoderState::Start => {
 					output.put_slice(XML_DECL);
 					self.state = EncoderState::Declared;
 					Ok(())
 				}
-				_ => Err(EncodeError::MisplacedXMLDeclaration),
+				_ => Err(EncodeError::MisplacedXmlDeclaration),
 			},
 			Item::ElementHeadStart(nsuri, local_name) => match self.state {
 				EncoderState::Start | EncoderState::Declared | EncoderState::Content => {
 					output.put_u8(b'<');
-					let (new, prefix) = self.ns.declare_auto(nsuri.clone());
+					let (_, prefix) = self.ns.declare_auto(nsuri.clone());
 					let qname = match prefix {
 						Some(prefix) => {
 							output.put_slice(prefix.as_bytes());
 							output.put_u8(b':');
+							output.put_slice(local_name.as_bytes());
 							prefix.with_suffix(local_name)
 						}
 						None => {
@@ -548,9 +655,15 @@ impl<T: TrackNamespace> Encoder<T> {
 						}
 					};
 					self.qname_stack.push(qname);
-					if new {
+					match self.ns.new_default_declaration() {
+						Some(name) => {
+							Self::encode_nsdecl(None, name.as_ref().map(|x| &****x), output)
+						}
+						None => (),
+					};
+					for (name, prefix) in self.ns.new_prefix_declarations() {
 						// if new, we have to declare it
-						Self::encode_nsdecl(prefix, nsuri.as_ref().map(|x| &***x), output);
+						Self::encode_nsdecl(Some(prefix), name.as_ref().map(|x| &***x), output);
 					}
 					self.state = EncoderState::ElementHead;
 					Ok(())
@@ -630,7 +743,7 @@ impl<T: TrackNamespace> Encoder<T> {
 	/// Encode a single item into a BytesMut.
 	///
 	/// This might have a slight performance advantage over
-	/// [`encode_into_bytes`] in some scenarios, as it might be able to give
+	/// [`encode`] in some scenarios, as it might be able to give
 	/// the BytesMut a heads up about required space, thus avoiding frequent
 	/// reallocations.
 	///
@@ -639,7 +752,7 @@ impl<T: TrackNamespace> Encoder<T> {
 	/// long-lived, buffers for serialization before sending data over the
 	/// network, for instance.
 	///
-	///   [`encode_into_bytes`]: Self::encode_into_bytes
+	///   [`encode`]: Self::encode
 	pub fn encode_into_bytes(
 		&mut self,
 		item: Item<'_>,
@@ -660,8 +773,8 @@ impl<T: TrackNamespace> Encoder<T> {
 		output: &mut O,
 	) -> Result<(), EncodeError> {
 		match ev {
-			ResolvedEvent::XMLDeclaration(_, version) => {
-				self.encode(Item::XMLDeclaration(*version), output)?;
+			ResolvedEvent::XmlDeclaration(_, version) => {
+				self.encode(Item::XmlDeclaration(*version), output)?;
 			}
 			ResolvedEvent::StartElement(_, (ns, name), attrs) => {
 				self.encode(Item::ElementHeadStart(ns.clone(), name.as_ref()), output)?;
@@ -732,12 +845,19 @@ mod tests_simple_namespaces {
 		assert!(prefix.is_none());
 	}
 
+	#[test]
+	fn predefined_prefixes_can_be_looked_up() {
+		let ns = mk();
+		assert_eq!(*ns.lookup_prefix(Some(PREFIX_XML)).unwrap(), XMLNS_XML);
+		assert_eq!(*ns.lookup_prefix(Some(PREFIX_XMLNS)).unwrap(), XMLNS_XMLNS);
+	}
+
 	#[test]
 	fn hardcoded_xmlns_namespace_for_declare_auto() {
 		let mut ns = mk();
 		let (new, prefix) = ns.declare_auto(Some(RcPtr::new(XMLNS_XMLNS.to_cdata())));
 		assert!(!new);
-		assert!(prefix.unwrap() == "xmlns");
+		assert!(prefix.unwrap() == PREFIX_XMLNS);
 	}
 
 	#[test]
@@ -745,7 +865,7 @@ mod tests_simple_namespaces {
 		let mut ns = mk();
 		let (new, prefix) = ns.declare_auto(Some(RcPtr::new(XMLNS_XML.to_cdata())));
 		assert!(!new);
-		assert!(prefix.unwrap() == "xml");
+		assert!(prefix.unwrap() == PREFIX_XML);
 	}
 
 	#[test]
@@ -820,6 +940,28 @@ mod tests_simple_namespaces {
 			Ok(None) => {}
 			other => panic!("unexpected get_prefix_or_default result: {:?}", other),
 		};
+		match ns.lookup_prefix(None) {
+			Ok(name) => {
+				assert_eq!(name, ns1());
+			}
+			other => panic!("unexpected lookup_prefix result: {:?}", other),
+		}
+	}
+
+	#[test]
+	fn declaration_causes_lookup_to_return_a_thing() {
+		let mut ns = mk();
+		match ns.lookup_prefix(None) {
+			Err(PrefixError::Undeclared) => (),
+			other => panic!("unexpected lookup_prefix result: {:?}", other),
+		}
+		ns.declare_auto(Some(ns1()));
+		match ns.lookup_prefix(None) {
+			Ok(name) => {
+				assert_eq!(name, ns1());
+			}
+			other => panic!("unexpected lookup_prefix result: {:?}", other),
+		}
 	}
 
 	#[test]
@@ -852,6 +994,26 @@ mod tests_simple_namespaces {
 		};
 	}
 
+	#[test]
+	fn declaration_causes_lookup_to_return_a_thing_for_a_prefix() {
+		let mut ns = mk();
+		match ns.lookup_prefix(None) {
+			Err(PrefixError::Undeclared) => (),
+			other => panic!("unexpected lookup_prefix result: {:?}", other),
+		}
+		let prefix = ns.declare_with_auto_prefix(Some(ns1())).1.to_ncname();
+		match ns.lookup_prefix(Some(&prefix)) {
+			Ok(name) => {
+				assert_eq!(name, ns1());
+			}
+			other => panic!("unexpected lookup_prefix result: {:?}", other),
+		}
+		match ns.lookup_prefix(None) {
+			Err(PrefixError::Undeclared) => (),
+			other => panic!("unexpected lookup_prefix result: {:?}", other),
+		}
+	}
+
 	#[test]
 	fn reuses_auto_allocated_prefix_for_same_ns() {
 		let mut ns = mk();
@@ -878,12 +1040,18 @@ mod tests_simple_namespaces {
 		assert!(!new);
 		assert!(prefix.is_none());
 		assert!(ns.get_prefix_or_default(Some(ns1())).unwrap().is_none());
+		match ns.lookup_prefix(None) {
+			Ok(name) => {
+				assert_eq!(name, ns1());
+			}
+			other => panic!("unexpected lookup_prefix result: {:?}", other),
+		}
 	}
 
 	#[test]
 	fn declare_with_fixed_prefix() {
 		let mut ns = mk();
-		ns.declare_fixed(Some(NCNameStr::from_str("stream").unwrap()), Some(ns1()));
+		ns.declare_fixed(Some(NcNameStr::from_str("stream").unwrap()), Some(ns1()));
 		match ns.get_prefix(Some(ns1())) {
 			Ok(v) => {
 				assert_eq!(v, "stream");
@@ -915,7 +1083,7 @@ mod tests_simple_namespaces {
 	#[test]
 	fn preserves_global_prefixed_across_elements() {
 		let mut ns = mk();
-		ns.declare_fixed(Some(NCNameStr::from_str("stream").unwrap()), Some(ns1()));
+		ns.declare_fixed(Some(NcNameStr::from_str("stream").unwrap()), Some(ns1()));
 		ns.declare_fixed(None, Some(ns2()));
 		ns.push();
 		match ns.get_prefix(Some(ns1())) {
@@ -944,10 +1112,10 @@ mod tests_simple_namespaces {
 	#[should_panic(expected = "conflict")]
 	fn prohibits_overriding_global_prefix() {
 		let mut ns = mk();
-		ns.declare_fixed(Some(NCNameStr::from_str("stream").unwrap()), Some(ns1()));
+		ns.declare_fixed(Some(NcNameStr::from_str("stream").unwrap()), Some(ns1()));
 		ns.declare_fixed(None, Some(ns2()));
 		ns.push();
-		ns.declare_fixed(Some(NCNameStr::from_str("stream").unwrap()), Some(ns2()));
+		ns.declare_fixed(Some(NcNameStr::from_str("stream").unwrap()), Some(ns2()));
 	}
 
 	#[test]
@@ -967,7 +1135,7 @@ mod tests_simple_namespaces {
 	#[should_panic(expected = "conflict")]
 	fn fixed_global_decl_can_conflict_with_auto_decl() {
 		let mut ns = mk();
-		ns.declare_fixed(Some(NCNameStr::from_str("tns0").unwrap()), Some(ns1()));
+		ns.declare_fixed(Some(NcNameStr::from_str("tns0").unwrap()), Some(ns1()));
 		ns.push();
 		ns.declare_with_auto_prefix(Some(ns3()));
 	}
@@ -976,7 +1144,7 @@ mod tests_simple_namespaces {
 	#[should_panic(expected = "conflict")]
 	fn fixed_local_decl_can_conflict_with_auto_decl() {
 		let mut ns = mk();
-		ns.declare_fixed(Some(NCNameStr::from_str("tns0").unwrap()), Some(ns1()));
+		ns.declare_fixed(Some(NcNameStr::from_str("tns0").unwrap()), Some(ns1()));
 		ns.declare_with_auto_prefix(Some(ns3()));
 	}
 }
@@ -985,6 +1153,8 @@ mod tests_simple_namespaces {
 mod tests_encoder {
 	use super::*;
 
+	use std::convert::TryFrom;
+
 	use crate::parser::EventMetrics;
 
 	use crate::EventRead;
@@ -993,6 +1163,18 @@ mod tests_encoder {
 		Encoder::new()
 	}
 
+	fn ns1() -> NamespaceName {
+		RcPtr::new(CData::try_from("uri:foo").unwrap())
+	}
+
+	fn ns2() -> NamespaceName {
+		RcPtr::new(CData::try_from("uri:bar").unwrap())
+	}
+
+	fn ns3() -> NamespaceName {
+		RcPtr::new(CData::try_from("uri:baz").unwrap())
+	}
+
 	fn parse(mut input: &[u8]) -> (Vec<ResolvedEvent>, crate::Result<bool>) {
 		let mut parser = crate::PullParser::new(&mut input);
 		let mut events = Vec::new();
@@ -1045,7 +1227,7 @@ mod tests_encoder {
 
 	fn assert_event_eq(a: &ResolvedEvent, b: &ResolvedEvent) {
 		match (a, b) {
-			(ResolvedEvent::XMLDeclaration(_, v1), ResolvedEvent::XMLDeclaration(_, v2)) => {
+			(ResolvedEvent::XmlDeclaration(_, v1), ResolvedEvent::XmlDeclaration(_, v2)) => {
 				assert_eq!(v1, v2);
 			}
 			(
@@ -1142,12 +1324,12 @@ mod tests_encoder {
 	fn reject_duplicate_xml_declaration() {
 		let mut enc = mkencoder();
 		let mut buf = BytesMut::new();
-		match enc.encode(Item::XMLDeclaration(XMLVersion::V1_0), &mut buf) {
+		match enc.encode(Item::XmlDeclaration(XmlVersion::V1_0), &mut buf) {
 			Ok(()) => (),
 			other => panic!("unexpected encode result: {:?}", other),
 		};
-		match enc.encode(Item::XMLDeclaration(XMLVersion::V1_0), &mut buf) {
-			Err(EncodeError::MisplacedXMLDeclaration) => (),
+		match enc.encode(Item::XmlDeclaration(XmlVersion::V1_0), &mut buf) {
+			Err(EncodeError::MisplacedXmlDeclaration) => (),
 			other => panic!("unexpected encode result: {:?}", other),
 		};
 	}
@@ -1199,8 +1381,8 @@ mod tests_encoder {
 			Ok(()) => (),
 			other => panic!("unexpected encode result: {:?}", other),
 		};
-		match enc.encode(Item::XMLDeclaration(XMLVersion::V1_0), &mut buf) {
-			Err(EncodeError::MisplacedXMLDeclaration) => (),
+		match enc.encode(Item::XmlDeclaration(XmlVersion::V1_0), &mut buf) {
+			Err(EncodeError::MisplacedXmlDeclaration) => (),
 			other => panic!("unexpected encode result: {:?}", other),
 		};
 	}
@@ -1223,6 +1405,165 @@ mod tests_encoder {
 		assert_eq!(&buf, &b"<x/>"[..]);
 	}
 
+	#[test]
+	fn encode_root_prefix() {
+		let mut enc = mkencoder();
+		let mut buf = BytesMut::new();
+		enc.ns
+			.declare_fixed(Some("foo".try_into().unwrap()), Some(ns1()));
+		match enc.encode(
+			Item::ElementHeadStart(Some(ns1()), "x".try_into().unwrap()),
+			&mut buf,
+		) {
+			Ok(()) => (),
+			other => panic!("unexpected encode result: {:?}", other),
+		};
+		match enc.encode(Item::ElementFoot, &mut buf) {
+			Ok(()) => (),
+			other => panic!("unexpected encode result: {:?}", other),
+		};
+		assert_eq!(&buf, &b"<foo:x xmlns:foo='uri:foo'/>"[..]);
+	}
+
+	#[test]
+	fn use_explicitly_set_at_root() {
+		let mut enc = mkencoder();
+		let mut buf = BytesMut::new();
+		enc.ns
+			.declare_fixed(Some("foo".try_into().unwrap()), Some(ns1()));
+		match enc.encode(
+			Item::ElementHeadStart(Some(ns2()), "x".try_into().unwrap()),
+			&mut buf,
+		) {
+			Ok(()) => (),
+			other => panic!("unexpected encode result: {:?}", other),
+		};
+		match enc.encode(Item::ElementHeadEnd, &mut buf) {
+			Ok(()) => (),
+			other => panic!("unexpected encode result: {:?}", other),
+		};
+		match enc.encode(
+			Item::ElementHeadStart(Some(ns1()), "y".try_into().unwrap()),
+			&mut buf,
+		) {
+			Ok(()) => (),
+			other => panic!("unexpected encode result: {:?}", other),
+		};
+		match enc.encode(Item::ElementFoot, &mut buf) {
+			Ok(()) => (),
+			other => panic!("unexpected encode result: {:?}", other),
+		};
+		match enc.encode(Item::ElementFoot, &mut buf) {
+			Ok(()) => (),
+			other => panic!("unexpected encode result: {:?}", other),
+		};
+		assert_eq!(
+			&buf,
+			&b"<x xmlns='uri:bar' xmlns:foo='uri:foo'><foo:y/></x>"[..]
+		);
+	}
+
+	#[test]
+	fn do_not_emit_duplicate_declarations() {
+		let mut enc = mkencoder();
+		let mut buf = BytesMut::new();
+		enc.ns
+			.declare_fixed(Some("foo".try_into().unwrap()), Some(ns1()));
+		match enc.encode(
+			Item::ElementHeadStart(Some(ns2()), "x".try_into().unwrap()),
+			&mut buf,
+		) {
+			Ok(()) => (),
+			other => panic!("unexpected encode result: {:?}", other),
+		};
+		match enc.encode(
+			Item::Attribute(
+				Some(ns1()),
+				"a1".try_into().unwrap(),
+				"v1".try_into().unwrap(),
+			),
+			&mut buf,
+		) {
+			Ok(()) => (),
+			other => panic!("unexpected encode result: {:?}", other),
+		}
+		match enc.encode(
+			Item::Attribute(
+				Some(ns1()),
+				"a2".try_into().unwrap(),
+				"v2".try_into().unwrap(),
+			),
+			&mut buf,
+		) {
+			Ok(()) => (),
+			other => panic!("unexpected encode result: {:?}", other),
+		}
+		match enc.encode(
+			Item::Attribute(
+				Some(ns2()),
+				"a3".try_into().unwrap(),
+				"v3".try_into().unwrap(),
+			),
+			&mut buf,
+		) {
+			Ok(()) => (),
+			other => panic!("unexpected encode result: {:?}", other),
+		}
+		match enc.encode(
+			Item::Attribute(
+				Some(ns2()),
+				"a4".try_into().unwrap(),
+				"v4".try_into().unwrap(),
+			),
+			&mut buf,
+		) {
+			Ok(()) => (),
+			other => panic!("unexpected encode result: {:?}", other),
+		}
+		match enc.encode(
+			Item::Attribute(
+				Some(ns3()),
+				"a5".try_into().unwrap(),
+				"v5".try_into().unwrap(),
+			),
+			&mut buf,
+		) {
+			Ok(()) => (),
+			other => panic!("unexpected encode result: {:?}", other),
+		}
+		match enc.encode(
+			Item::Attribute(
+				Some(ns3()),
+				"a6".try_into().unwrap(),
+				"v6".try_into().unwrap(),
+			),
+			&mut buf,
+		) {
+			Ok(()) => (),
+			other => panic!("unexpected encode result: {:?}", other),
+		}
+		match enc.encode(Item::ElementHeadEnd, &mut buf) {
+			Ok(()) => (),
+			other => panic!("unexpected encode result: {:?}", other),
+		};
+		match enc.encode(
+			Item::ElementHeadStart(Some(ns1()), "y".try_into().unwrap()),
+			&mut buf,
+		) {
+			Ok(()) => (),
+			other => panic!("unexpected encode result: {:?}", other),
+		};
+		match enc.encode(Item::ElementFoot, &mut buf) {
+			Ok(()) => (),
+			other => panic!("unexpected encode result: {:?}", other),
+		};
+		match enc.encode(Item::ElementFoot, &mut buf) {
+			Ok(()) => (),
+			other => panic!("unexpected encode result: {:?}", other),
+		};
+		assert_eq!(&buf, &b"<x xmlns='uri:bar' xmlns:foo='uri:foo' foo:a1=\"v1\" foo:a2=\"v2\" xmlns:tns0='uri:bar' tns0:a3=\"v3\" tns0:a4=\"v4\" xmlns:tns1='uri:baz' tns1:a5=\"v5\" tns1:a6=\"v6\"><foo:y/></x>"[..]);
+	}
+
 	#[test]
 	fn self_closed_tag_encoding_ends_document_if_it_closes_the_root() {
 		let mut enc = mkencoder();
